Sequential

Struct Sequential 

pub struct Sequential<State, Transition, StateStrategy, TransitionStrategy> { /* private fields */ }
In a sequential state machine strategy, we first generate an acceptable sequence of transitions. That is a sequence that satisfies the given pre-conditions. The acceptability of each transition in the sequence depends on the current state of the state machine, which is updated by the transitions with the next function.

The shrinking strategy is to iteratively apply Shrink::InitialState, Shrink::DeleteTransition and Shrink::Transition.

  1. We start by trying to delete transitions from the back of the list that were never seen by the test, if any. Note that because proptest expects deterministic results in for reproducible issues, unlike the following steps this step will not be undone on complicate. If there were any unseen transitions, then the next step will start at trying to delete the transition before the last one seen as we know that the last transition cannot be deleted as it’s the one that has failed.
  2. Then, we keep trying to delete transitions from the back of the list, until we can do so no further (reached the beginning of the list)..
  3. Then, we again iteratively attempt to shrink the individual transitions, but this time starting from the front of the list - i.e. from the first transition to be applied.
  4. Finally, we try to shrink the initial state until it’s not possible to shrink it any further.

For complicate, we attempt to undo the last shrink operation, if there was any.

Implementations

Source

impl<State, Transition, StateStrategy, TransitionStrategy> Sequential<State, Transition, StateStrategy, TransitionStrategy>
where State: 'static, Transition: 'static, StateStrategy: 'static, TransitionStrategy: 'static,

pub fn new( size: SizeRange, init_state: impl Fn() -> StateStrategy + 'static + Send + Sync, preconditions: impl Fn(&State, &Transition) -> bool + 'static + Send + Sync, transitions: impl Fn(&State) -> TransitionStrategy + 'static + Send + Sync, next: impl Fn(State, &Transition) -> State + 'static + Send + Sync, ) -> Self